# -*- coding: utf-8 -*-


import unittest
from dicts import SortedDict



class SortedDictTestCase(unittest.TestCase):
    """SortedDict test cases."""

    def setUp(self):
        self.sd = SortedDict({'c' : (1,'b'), 'a' : (6,'c'), 'b' : (5,'a')},
                             key=lambda x: x[1][1])

    def tearDown(self):
        pass

    def test_items(self):
        self.assertEqual(self.sd.items(),
                         [('b', (5, 'a')), ('c', (1, 'b')), ('a', (6, 'c'))])

    def test_keys(self):
        self.assertEqual(self.sd.keys(), ['b', 'c', 'a'])

    def test_values(self):
        self.assertEqual(self.sd.values(), [(5, 'a'), (1, 'b'), (6, 'c')])

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

"""
 Dictiories that are sorted when they are iterated

 KeySortedDict:

 >>> ksd = KeySortedDict({'c' : 1, 'a' : 6, 'b' : 5})
 >>> ksd.items()
 [('a', 6), ('b', 5), ('c', 1)]
 >>> ksd.keys()
 ['a', 'b', 'c']
 >>> ksd.values()
 [6, 5, 1]

 ValueSortedDict

 >>> vsd = ValueSortedDict({'c' : 1, 'a' : 6, 'b' : 5})
 >>> vsd.items()
 [('c', 1), ('b', 5), ('a', 6)]
 >>> vsd.keys()
 ['c', 'b', 'a']
 >>> vsd.values()
 [1, 5, 6]

 SortedDict

 >>> sd = SortedDict({'c' : (1,'b'), 'a' : (6,'c'), 'b' : (5,'a')}, key=lambda x: x[1][1])
 >>> sd.items()
 [('b', (5, 'a')), ('c', (1, 'b')), ('a', (6, 'c'))]

 author : Miguel Olivares <miguel@moliware.com>
"""

from dict import Dict

class SortedDict(Dict):
    """ Dictionary that iterates over its elements """

    def __init__(self, data={}, cmp=None, key=None, reverse=False):
        super(SortedDict, self).__init__(data)
        self.cmp = cmp
        self.key = key
        self.reverse= reverse

    def __iter__(self):
        """ Sort and then iterate the dictionary """
        sorted_data = sorted(self.data.iteritems(), self.cmp, self.key,
                             self.reverse)
        for k,v in sorted_data:
            yield k,v

    def iteritems(self):
        for x in self.__iter__():
            yield x

    def items(self):
        return [x for x in self.iteritems()]

    def iterkeys(self):
        for k,v in self.__iter__():
            yield k

    def keys(self):
        return [k for k in self.iterkeys()]

    def itervalues(self):
        for k,v in self.__iter__():
            yield v

    def values(self):
        return [v for v in self.itervalues()]

class KeySortedDict(SortedDict):
    """ Dictionary sorted by key """
    def __init__(self, data={}, reverse=False):
        """ """
        super(KeySortedDict, self).__init__(data, reverse=reverse)

class ValueSortedDict(SortedDict):
    """ Dictionary sorted by value """
    def __init__(self, data={}, reverse=False):
        """ """
        super(ValueSortedDict, self).__init__(data, key=lambda x:x[1],
                                               reverse=reverse)


# -*- coding: utf-8 -*-

"""
 Keys of the dictionary are case insensitive.

 Command line Usage:

 >>> ncd = NoCaseDict({'examPLE' : 1})
 >>> 'example' in ncd
 True

 author : Miguel Olivares <miguel@moliware.com>
"""

from dict import Dict


class NoCaseDict(Dict):
    """ Keys of the dictionary are case insensitive. """

    def __init__(self, data):
        correct_keys = reduce(lambda x, y: x and isinstance(y, basestring),
                              data.iterkeys(), True)
        if not correct_keys:
            raise KeyError
        super(NoCaseDict, self).__init__(dict(map(lambda x : (x[0].lower(), x[1]), data.items())))

    def __setitem__(self, key, value):
        if not isinstance(key, basestring):
            raise KeyError
        super(NoCaseDict, self).__setitem__(key.lower(), value)

    def __contains__ (self, key):
        if not isinstance(key, basestring):
            raise KeyError
        return super(NoCaseDict, self).__contains__(key.lower())
        


# -*- coding: utf-8 -*-

"""
 Extends UserDict to implement some awesome features

 iterable:

 >>> d = Dict({1 : 2, 2 : 3})
 >>> for k,v in d:
 >>>     print k,v
 1 2
 2 3

 join:

 >>> d = Dict({1 : 2})
 >>> d.join({1 : 3})
 {1: 5}

 >>> d = Dict({1 : 2})
 >>> d.join({1 : [3]})
 {1: [2, 3]}

 author : Miguel Olivares <miguel@moliware.com>
"""
from copy import deepcopy
from UserDict import UserDict

class Dict(UserDict, object):
    """ Iterable UserDict. It is thought for being
    the base class of all specific dictionaries.
    """

    def __iter__(self):
        """ Iterable dictionary! """
        return self.data.iteritems()

    def join(self, dic):
        """ Add dic pairs to self.data """
        for k,v in dic.iteritems():
            if k in self.data:
                self[k] += dic[k]
            else:
                self[k] = deepcopy(v)
        return self

    def map(self, callable):
        """ Apply 'callable' function over all values. """
        for k,v in self.iteritems():
            self[k] = callable(v)

    @classmethod
    def fromrepetitions(cls, iterable):
        """ Create a dict whose keys are the members of the iterable and values
        are the number of times the key appears in the iterable.
        """
        d = cls()
        for key in iterable:
            d[key] = d[key] + 1 if key in d else 1
        return d
